Program PicSlice;
{ Converts an image in the RAW format, i.e. one of which each byte
| represents one gray pixel, no header, into two files, one for the
| black-and-white image plane, the other for the gray-and-white image
| plane. These files may eg serve as inputs for PICWRITE.EXE which
| produces an image in the .PIC format readable by the software of
| the palmtop computer PSION 3C.
|
| Format:        > PicSlice input_file
|
| The two output files will have the same file name as the input file
| but the extensions .BLK and .GRY.
|
| Peter M. W. Nave',   1999-03-05,   latest revision 1999-03-06
| Turbo Pascal 6, 100010.3276@CompuServe.com   *** FreeWare ***
|----------------------------------------------------------------------}
  uses
     CRT;
  var
     filein, fileoutb, fileoutg, tempstr: string;
     fibi, fibob, fibog: file of byte;
     lowthr, uppthr, tb: byte;
     tk, tl : longint;
     ii: integer;
  const
     one: byte = 1;   zero: byte = 0;

  procedure Quit;
  begin
     Writeln(' >>> Press any key to finish.');
     repeat until Keypressed; Halt;
  end;

  procedure PicHelp;
  const
     textlength = 17;
     sstt: array[1 .. textlength] of string[72] = (
        ' ',
        'PicSlice converts an image in the RAW format, i.e. one of ',
        'which each byte represents one gray-scale pixel, no header,',
        'into two files, one for the black-and-white image plane, ',
        'the other for the gray-and-white image  plane. These files',
        'may eg serve as inputs for PICWRITE.EXE which produces an',
        'image in the .PIC format readable by the software of the',
        'palmtop computer PSION 3C.',
        ' ',
        'Format:        > PicSlice input_file',
        ' ',
        'The two output files will have the same file name as the',
        'input file but the extensions .BLK and .GRY.  Note that',
        'pixel values above the upper threshold are mapped into 1 ',
        'in the black file, values between lower threshold and upper',
        'threshold are mapped into 1 of the gray file and pixels',
        'below the lower level are set to zero in both output files.');
  var
     ii: integer;
  begin
     for ii := 1 to textlength do
        Writeln('     ', sstt[ii]);
     Quit;
  end;

  begin
     Writeln; Writeln(' --- PicSlice');
     if ParamCount = 0 then PicHelp
     else filein := ParamStr(1);
     for ii := 1 to Length(filein) do filein[ii] := UpCase(filein[ii]);
     Writeln(' --- Input file is:  ', filein);
{$I-}
     Assign(fibi, filein); Reset(fibi);
     if IOResult <> 0 then begin
        Writeln(' ### File ', filein, ' not found.');
        Quit;
        end;
{$I+}
     ii := Pos('.', filein);
     if ii > 0 then begin
        if Length(filein) = ii + 3 then begin
           if Copy(filein, ii, 4) = '.PIC' then begin
              Writeln(' ### Change extension of input file name!');
              Quit;
              end;
           end;
        tempstr := Copy(filein, 1, ii - 1) ;
        end
     else
        tempstr := filein;
     fileoutb := tempstr + '.BLK';
     fileoutg := tempstr + '.GRY';

     Assign(fibob, fileoutb); Rewrite(fibob);
     Assign(fibog, fileoutg); Rewrite(fibog);
     Writeln(' --- Output files are: ', fileoutb, ' and ', fileoutg);

     lowthr := 0;
     while (lowthr <= 0) or (lowthr >= 254) do begin
        Write(' >>> Enter the lower slicing threshold ',
              '( > 0,  < 255):  ');
        Readln(lowthr);
        end;
     uppthr := 0;
     while (uppthr <= lowthr) or (uppthr >= 254) do begin
        Write(' >>> Enter the upper slicing threshold ( > ', lowthr,
              ', < 255):  ');
        Readln(uppthr);
        end;

     tl := FileSize(fibi);
     WriteLn(' --- There are ', tl, ' bytes in the file');
{
| No attempt is made here to speed up the IO-process because the image
| files are usaually small and present day computers are very fast.
}
     for tk := 1 to tl do begin
        Read(fibi, tb);
        if tb <= lowthr then begin
           Write(fibob, zero);
           Write(fibog, one);
           end
        else if tb <= uppthr then begin
           Write(fibob, one);
           Write(fibog, zero);
           end
        else begin
           Write(fibob, one);
           Write(fibog, one);
           end;
        end;
   
     Close(fibi); Close(fibob); Close(fibog);
     Quit;
  end.