  program PicWrite;
{
| Converts a binary valued, i. e. black-and-white only, image in the RAW
| format, i. e. all bytes are image pixels, no header, to an image in
| the .PIC format of the PSION Series 3c. In the RAW image, 0 is assumed
| to be black, every byte > 0 is taken to be white. In the PIC format, a
| set bit is black, a clear bit is white. The input file can contain
| more than one bitmap of varying size. Prior to calling PicWrite,
| concatenate individual bitmaps with the DOS command:
|
|              copy /b file_1 + file_2 + ...  file_all /v
|
| Syntax:      > PICWRITE filename
|              and answer all questions which appear on the screen.
|
| Calling PicWrite with no command line parameter displays this text.
| Code: Turbo Pascal 5.0.
|
| P. M. W. Nave, 1997-06-02, latest revision: 1999-03-05
| *** FreeWare ***    e-mail: 100010.3276@CompuServe.com
|----------------------------------------------------------------------}
  uses
     CRT;
  var
     filein, fileout: string;
     tm : array[1..MaxInt] of byte;
     ii, jj, mm : integer;
     fibi, fibm, fibo : file of byte;
     rows, cols, crcp, colp, tw, maps : word;
     tb: byte;
     tk, tl : longint;
     bb : array[1..4] of byte absolute tl;
     varsize : boolean;
     cc : char;

  procedure Quit;
  begin
     Writeln(' >>> Press any key to finish.');
     repeat until Keypressed; Halt;
  end;

  procedure PicHelp;
  const
     textlength = 21;
     sstt : array[1 .. textlength] of string[72] = (
        ' ',
        'converts a binary valued, i. e. black-and-white only, image ',
        'in the RAW format, i. e. all bytes are image pixels, no ',
        'header, to an image in the PIC format of the PSION Series 3c.',
        'In the RAW image, 0 is assumed to be black, every byte > 0 is',
        'taken to be white. In the PIC format, a set bit is black, a ',
        'clear bit is white. The input file can contain more than one',
        'bitmap of varying size. Prior to calling PicWrite, ',
        'concatenate individual bitmaps with the DOS command:',
        ' ',
        '       copy /b file_1 + file_2 + ...  file_all /v',
        ' ',
        'FILE_ALL would then be the input file to PicWrite.',
        ' ',
        'Syntax:      > PICWRITE filename',
        '             and answer all questions which appear on the ',
        '             screen.',
        ' ',
        'Calling PicWrite with no command line parameter displays ',
        'this text.',
        ' ');
     begin
        for ii := 1 to textlength do
           Writeln('     ', sstt[ii]);
        Quit;
     end;

  procedure BinComp(var vector; var size: word);
{
| Compresses the SIZE bytes of VECTOR to bits.
}
  type
     bytearr = array[1..MaxInt] of byte;
  var
     ii, jj, kk: integer;
     hold, tb : byte;
  begin
     jj := 0; hold := 0; kk := 1;
     for ii := 1 to size do begin
        tb := bytearr(vector)[ii];
        if tb > 0 then tb := 0 else tb := 1;
        hold := hold or (tb shl jj);
        Inc(jj);
        if jj = 8 then begin
           bytearr(vector)[kk] := hold;
           hold := 0;
           jj := 0;
           if ii < size then Inc(kk);
           end;
        end;
     if jj <> 0 then bytearr(vector)[kk] := hold;
     if Odd(kk) then begin
        Inc(kk); bytearr(vector)[kk] := 0;
        end;
     size := kk;
  end;

  function CRC(var vector; size, c16: word): word;
{
| The 16-bits cyclic redundancy code C16 is updated by the elements of
| byte array VECTOR to become CRC. VECTOR may be called with a subscript
| for the first element to be used, e.g.:
|
|                   temp := CRC(vector[20], 80, c16);
|
| Here 80 bytes of the array VECTOR, starting with element 20, will be
| processed. The mechanics of untyped parameters is described in:
| Borland: Turbo Pascal for the Mac. User's Guide and Reference Manual,
| Borland International Inc, Scotts Valley, CA, USA, ISBN 0-87524-154-9,
| 1986, page 264.
|----------------------------------------------------------------------}
  const
     CRC16Tab: array[0 .. 255] of word = (
        $0000, $1021, $2042, $3063, $4084, $50A5, $60C6, $70E7,
        $8108, $9129, $A14A, $B16B, $C18C, $D1AD, $E1CE, $F1EF,
        $1231, $0210, $3273, $2252, $52B5, $4294, $72F7, $62D6,
        $9339, $8318, $B37B, $A35A, $D3BD, $C39C, $F3FF, $E3DE,
        $2462, $3443, $0420, $1401, $64E6, $74C7, $44A4, $5485,
        $A56A, $B54B, $8528, $9509, $E5EE, $F5CF, $C5AC, $D58D,
        $3653, $2672, $1611, $0630, $76D7, $66F6, $5695, $46B4,
        $B75B, $A77A, $9719, $8738, $F7DF, $E7FE, $D79D, $C7BC,
        $48C4, $58E5, $6886, $78A7, $0840, $1861, $2802, $3823,
        $C9CC, $D9ED, $E98E, $F9AF, $8948, $9969, $A90A, $B92B,
        $5AF5, $4AD4, $7AB7, $6A96, $1A71, $0A50, $3A33, $2A12,
        $DBFD, $CBDC, $FBBF, $EB9E, $9B79, $8B58, $BB3B, $AB1A,
        $6CA6, $7C87, $4CE4, $5CC5, $2C22, $3C03, $0C60, $1C41,
        $EDAE, $FD8F, $CDEC, $DDCD, $AD2A, $BD0B, $8D68, $9D49,
        $7E97, $6EB6, $5ED5, $4EF4, $3E13, $2E32, $1E51, $0E70,
        $FF9F, $EFBE, $DFDD, $CFFC, $BF1B, $AF3A, $9F59, $8F78,
        $9188, $81A9, $B1CA, $A1EB, $D10C, $C12D, $F14E, $E16F,
        $1080, $00A1, $30C2, $20E3, $5004, $4025, $7046, $6067,
        $83B9, $9398, $A3FB, $B3DA, $C33D, $D31C, $E37F, $F35E,
        $02B1, $1290, $22F3, $32D2, $4235, $5214, $6277, $7256,
        $B5EA, $A5CB, $95A8, $8589, $F56E, $E54F, $D52C, $C50D,
        $34E2, $24C3, $14A0, $0481, $7466, $6447, $5424, $4405,
        $A7DB, $B7FA, $8799, $97B8, $E75F, $F77E, $C71D, $D73C,
        $26D3, $36F2, $0691, $16B0, $6657, $7676, $4615, $5634,
        $D94C, $C96D, $F90E, $E92F, $99C8, $89E9, $B98A, $A9AB,
        $5844, $4865, $7806, $6827, $18C0, $08E1, $3882, $28A3,
        $CB7D, $DB5C, $EB3F, $FB1E, $8BF9, $9BD8, $ABBB, $BB9A,
        $4A75, $5A54, $6A37, $7A16, $0AF1, $1AD0, $2AB3, $3A92,
        $FD2E, $ED0F, $DD6C, $CD4D, $BDAA, $AD8B, $9DE8, $8DC9,
        $7C26, $6C07, $5C64, $4C45, $3CA2, $2C83, $1CE0, $0CC1,
        $EF1F, $FF3E, $CF5D, $DF7C, $AF9B, $BFBA, $8FD9, $9FF8,
        $6E17, $7E36, $4E55, $5E74, $2E93, $3EB2, $0ED1, $1EF0);
  type
     bytearr = array[1 .. maxint] of byte;
  var
     ii: integer;
  begin
     for ii := 1 to size do
        c16 := (c16 shl 8) xor (CRC16tab[(c16 shr 8)
                           xor bytearr(vector)[ii]]);
     CRC := c16;
  end;

  begin
     Writeln; Writeln(' --- PicWrite');
     if ParamCount > 0 then filein := ParamStr(1)
     else PicHelp;
{$I-}
     Assign(fibi, filein); Reset(fibi);
     if IOResult <> 0 then begin
        Writeln(' ### File ', filein, ' not found.');
        PicHelp;
        end;
     for ii := 1 to Length(filein) do filein[ii] := UpCase(filein[ii]);
     Writeln(' --- Input file is:  ', filein);
{$I+}
     tw := Pos('.', filein);
     if tw > 0 then
        if Length(filein) = tw + 3 then
           if Copy(filein, tw, 4) = '.PIC' then begin
              Writeln(' ### Change extension of input file name!');
              PicHelp;
              end;
     if tw = 0 then fileout := filein + '.PIC'
     else fileout := Copy(filein, 1, tw) + 'PIC';

     Assign(fibo, fileout); Rewrite(fibo);
     Writeln(' --- Output file is: ', fileout);

     Write(' >>> Enter the number of bitmaps in the input: ');
     Readln(maps);
     varsize := false;
     if maps = 1 then
        Write(' >>> Enter the number of rows in the input file: ')
     else begin
        while KeyPressed do cc := ReadKey;
        Write(' >>> Are the bitmaps of different sizes? [Y/N]:  ');
        cc := Upcase(ReadKey);
        WriteLn(cc);
        varsize := ((cc = 'Y') or (cc = 'J') or (cc = 'O'));
        if not varsize then
           Write(' >>> Enter the number of rows in the input files: ');
        end;
     if not varsize then begin
        Readln(rows);
        tl := FileSize(fibi);
        cols := (tl div maps) div rows;
        tk := longint(rows) * cols * maps;
        if tk <> tl then begin
           Writeln(' ### PICWRITE: wrong file size: ');
           Writeln('     FileSize = ', tl,
              ', rows * columns * maps = ', rows, ' * ', cols, ' * ',
              maps, ' = ', tk, '.');
           Close(fibi); Close(fibo);
           Quit;
           end;
        Write(' --- There are ', rows, ' rows and ', cols,
             ' columns in ');
        if maps = 1 then Writeln(filein, '.')
        else Writeln('each bitmap.');
        end;

     Assign(fibm, 'c:scratch.tmp'); Rewrite(fibm);

{ Mandatory beginning of header.}
     tm[1] := 80; tm[2] := 73; tm[3] := 67; tm[4] := 220;
     tm[5] := 48; tm[6] := 48;

{ Number of bitmaps in file. }
     tm[7] := Lo(maps);  tm[8] := Hi(maps);

     for ii := 1 to 8 do Write(fibo, tm[ii]);

     for mm := 1 to maps do begin
        crcp := 0;
        if varsize then begin
           Write(' >>> Enter the number of rows in bitmap no. ', mm,
              ': ');
           Readln(rows);
           Write(' >>> Enter the number of columns in bitmap no. ', mm,
              ': ');
           Readln(cols);
           end;
        for ii := 1 to rows do begin
           for jj := 1 to cols do Read(fibi, tm[jj]);
           colp := cols;
           BinComp(tm, colp);
           crcp := CRC(tm, colp, crcp);
           for jj := 1 to colp do Write(fibm, tm[jj]);
           end;

{ CRC of bitmap. }
        tm[1] := Lo(crcp); tm[2] := Hi(crcp);
{ Width and height of bitmap. }
        tm[3] := Lo(cols); tm[4] := Hi(cols);
        tm[5] := Lo(rows); tm[6] := Hi(rows);
{ Size of bitmap in bytes. }
        tw := colp * rows;
        if ((mm = 1) and (not varsize)) then begin
           Writeln(' --- There are ', rows, ' rows and ', colp,
                ' bytes per column in each bitmap of ', fileout, ',');
           Writeln('     each bitmap is ', tw, ' bytes long.');
           end
        else begin
           Writeln(' --- There are ', rows, ' rows and ', colp,
                ' bytes per column in bitmap no. ', mm, ' of ',
                fileout, ',');
           Writeln('     the bitmap is ', tw, ' bytes long.');
           end;

        tm[7] := Lo(tw); tm[8] := Hi(tw);
{ Distance to start of bitmap. }
        tl := longint(maps - mm) * 12 + longint(mm - 1) * tw;
        for jj := 1 to 4 do tm[8 + jj] := bb[jj];
        for jj := 1 to 12 do Write(fibo, tm[jj]);
        end;

     Writeln(' --- Header written.');
     Close(fibi); Reset(fibm);
     tl := maps * longint(tw);
     for tk := 1 to tl do begin
        Read(fibm, tb); Write(fibo, tb);
        end;
     if maps = 1 then Writeln(' --- Bitmap written.')
     else Writeln(' --- ', maps, ' bitmaps written.');

     Close(fibm); Erase(fibm); Close(fibo);
     Quit;
  end.